/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Error bar calculation using the blocking analysis                   //
//                                                                     //
// Burkhard Militzer                                   Urbana 12-13-99 //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

#ifndef _BLOCKER_
#define _BLOCKER_

// calculates the average and the blocked error bar for y(n1<=i<n2, col)
// returns the average in av, the blocked errorbar in er and the unblocked errorbar in er1
// y is not modified
inline void Blocker(const Array1 <double> & y, const int n1, const int n2, double & av, double & er, double & er1, const bool print=false) {
  double err;

  int n = n2-n1;
  if (n<=0) error( "No data given to blocker",n1,n2);

  //  Write4("Blocker",n1,n2,n);

  if (n==1) {
    av  = y(n1);
    er1 = 0.0;
    er  = 0.0;
    return;
  }

  // make a copy to make live easier
  Array1 <double> x(n);
  for(int i=0; i<n; i++) {
    x[i] = y[i+n1];
  }

  int nn = n; // number of blocks
  int l  = 1; // length of block
  do {
    double sx  = 0.0;
    double sx2 = 0.0;
    for(int i=0; i<nn; i++) {
      sx  += x[i];
      sx2 += x[i]*x[i];
    }
    sx  /= double(nn);
    sx2 /= double(nn);
    double d  = max(0.0,sx2-sx*sx); // could become -10^-16
    double sd = sqrt(d);
    err  = d / double(nn-1);
    err *= double(l*nn)/double(n); // compensate for dropped points at end
    err  = sqrt(err);

    if (print) cout << " Block_length= " << l << " number_of_blocks= " << nn << " <x>= " << sx << " std_dev= " << sd << " err= " << err << endl;

    if (l==1) { 
      av = sx;
      er1= err;
    };
    nn /= 2;
    l  *= 2;
    for(int i=0;i<nn;i++) {
      x[i] = (x[2*i]+x[2*i+1])*0.5;
    }
  }  while (nn>=9);
  er = err;
  if (print) cout << " Final result <x>= " << av << " +- " << err << endl;
}

// calculates the average and the blocked error bar for y(n1<=i<n2, col)
// returns the average in av, the blocked errorbar in er and the unblocked errorbar in er1
// y is not modified
void BlockerFast(const Array1 <double> & y, const int n1, const int n2, double & av, double & err) {
  const int n = n2-n1;
  if (n<=0) error( "No data given to blocker",n1,n2);

  if (n==1) {
    av  = y(n1);
    err = 0.0;
    return;
  }

  av = 0.0; // compare average first
  for(int i=n1; i<n2; i++) {
    av += y[i];
  }
  av /= double(n);

  const int minimumBlockNumber = 9;     // copied over from David Ceperley
  const int length = n / minimumBlockNumber; // block length, integer division

  Array1 <double> x(length);
  for(int i=0; i<minimumBlockNumber; i++) {
    x[i] = 0.0;
    for(int j=0; j<length; j++) {
      x[i] += y[n1+i*length+j];
    }
    x[i] /= double(length); // average of block 'i'
  }

  double sx  = 0.0;
  double sx2 = 0.0;
  for(int i=0; i<minimumBlockNumber; i++) {
    sx  += x[i];
    sx2 += x[i]*x[i];
  }
  sx  /= double(minimumBlockNumber);
  sx2 /= double(minimumBlockNumber);
  err  = max(0.0,sx2-sx*sx) / double(minimumBlockNumber-1); // could become -10^-16
  err *= double(length*minimumBlockNumber)/double(n);
  err  = sqrt(err);
}

#endif // _BLOCKER_
