/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//  Monte Carlo routines                                                   //
//                                                                         //
//  B. Militzer                                     Berkeley, CA, 02-09-20 //
//                                                                         //
/////////////////////////////////////////////////////////////////////////////

#ifndef _MC_
#define _MC_

#include "AccRatio.h"

template <class S> 
void PerformOneMonteCarloSweep(S & s, const double temp) {
  const bool print = false;

  //  cout << endl << " **************************************" << endl;
  if (print) cout << endl << " Starting a Monte Carlo sweep over " << s.nDim() << " parameters." << endl << endl;

  if (!s.Valid()) {  // should not happen
    cout << "Error: Current parameters set not valid: " << s << endl;
    error("Current parameters set not valid.");
  }

  double chi2 = s(); // I think we need to recompute this at the beginning of every sweep

  int nInvalid = 0;
  for(int i=0; i<s.nDim(); i++) { // loop pover all parameters

    double pOrg = s.Parameter(i);
    //    s.Parameter(i) += s.ParameterStep(i)*(2.0*Random()-1);
    s.Parameter(i) = s.Parameter(i) + s.ParameterStep(i)*(2.0*Random()-1);

    if (!s.Valid()) {
      if (print) cout << "Parameter change " << i << " not valid: from " << pOrg << " to " << s.Parameter(i) << endl;
      s.Parameter(i)=pOrg;
      nInvalid++;
      if (nInvalid>100) error("Something went wrong. Parameters repeatedly invalid.",s);
    } else {
      nInvalid = 0;
      bool ok = true;
      double chi2New;
      try {
	chi2New = s();
      } 
      catch(const char* ss) {
	cout << ss << endl;
	s.Reset();
	ok = false;
      }
	//	if (ok) Write5(i,chi2,chi2New,pOrg,mJ2.p.Parameter(i));
      if (!ok && print) { cout << "Not ok:"; Write5(i,chi2,chi2New,pOrg,s.Parameter(i)); }
	
      const string ss = " :::: ";

      //	double chi2RelTol = 0.05;
      //	if (ok && chi2New<chi2*(1.0+chi2RelTol)) {
      //       	double chi2Tol = 0.5;
      //	if (ok && chi2New<chi2+chi2Tol) {
      double diffChi2 = chi2New - chi2;
      
      if (ok && exp(-diffChi2/temp)>Random()) {
	if (print) cout << endl;
	//	  Write2(chi2,chi2New);
	if (print) { cout << ss << "ACCEPTED" << ss << " " << s; Write3(i,chi2,chi2New); } // cout << " " << mJ2.p << endl;
	if (print) { cout << " A: i= " << i << " " << s << " para= " << s.Parameter(i) << " step= " << s.ParameterStep(i) << endl; }
	chi2 = chi2New;
	//	s.ParameterStep(i) *= 1.2; // dangerous
	//	cout << mJ2.p;
      } else {
	s.Parameter(i) = pOrg;
	if (print) cout << endl;
	if (print) { cout << " R: i= " << i << " " << s << " para= " << s.Parameter(i) << " step= " << s.ParameterStep(i) << endl; }
	//	s.ParameterStep(i) *= 0.8; // dangerous
	//	cout << mJ2.p;
	if (print) { cout << ss << "REJECTED" << ss << " " << s; Write3(i,chi2,chi2New); } // cout << " " << mJ2.p << endl; 
      }

      if (print) { cout << ss << " STATE  " << ss; write1(chi2); cout << " " << s << endl; }
      //      mJ2.cms.WriteLayerEOSDataToFile(""); // write core mass etc. to standard out
      //      mJ2.cms.PrintBarotropeInfo();        // print P1,T1,P2,T2 and Ledoux
    }
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

// return values distributed like 1/sqrt(z) between 1/a and a
inline double SampleZ(const double a) {
  double x = Random();
  double d = 1.0/(1.0-a);
  double c = 1.0/(a*d*d);
  double z = c*sqr(x-d);
  return z;
}

inline double SampleZUniform(const double a) {
  double a1 = 1.0/a;
  double z = a1+Random()*(a-a1);
  return z;
}

template <class S> 
void PerformOneAffineMonteCarloSweep(Array1 <S> & s, AccRatio & accRatio, const double a, const int nWalkers, const double temp) {
  const bool print = false;
  if (nWalkers<3) error("nWalkers too small",nWalkers);

  //  cout << endl << " **************************************" << endl;
  if (print) cout << endl << " Starting affine Monte Carlo sweep with " << nWalkers << " walkers." << endl << endl;

  S sNew = s[0]; 
  const int nDim = sNew.nDim();

  int nInvalid = 0;
  for(int k=0; k<nWalkers; k++) { // loop over all walkers
    int j = int(double(nWalkers-1)*Random());
    if (j>=k) j++; // shift some j to avoid j==k

    // z in [0.5,2.0]
    // sNew = s[j] + z * (s[k]-s[j]) = z*s[k] + (1-z)*s[j]
    // z=1/2 --> sNew = s[j]+1/2*(s[k]-s[j]) = 0.5*s[k] + 0.5*s[j]
    // z=1   --> sNew = s[j]+    (s[k]-s[j]) = 1.0*s[k]
    // z=2   --> sNew = s[j]+ 2 *(s[k]-s[j]) = 2.0*z[k] -     s[j]

    //    double z = SampleZUniform(4.0); // test 6/14/21
    //    double z = SampleZ(2.0);
    //    double z = SampleZ(2.5);
    double z = SampleZ(a);

    for(int i=0; i<nDim; i++) {
      sNew.Parameter(i) = s[j].Parameter(i) + z * ( s[k].Parameter(i) - s[j].Parameter(i) ); 
    }

    accRatio.Try();
    if (!sNew.Valid()) {

      cout << "Problem: Invalid are the new parameters for walker " << k << endl;
      nInvalid++;
      if (nInvalid>100) error("Something went wrong. Parameters repeatedly invalid.",sNew);

    } else {

      nInvalid = 0;
      bool ok = true;
      double yNew;
      try {
	yNew = sNew.Evaluate();
	if (print) Write5(k,j,yNew,s[k].y,s[j].y);
      } 
      catch(const char* message) {
	cout << message << endl;
	sNew.Reset();
	ok = false;
	cout << "Not ok:"; Write4(k,j,z,sNew); 
      }
      //	if (ok) Write5(i,chi2,chi2New,pOrg,mJ2.p.Parameter(i));
    
      const string ss = " :::: ";
      const double dy = yNew - s[k].y;

      double acceptanceProbability = pow(z,nDim-1)*exp(-dy/temp); // correct if combined with SampleZ(2.0);
      //      double acceptanceProbability = pow(z,nDim-2)*exp(-dy/temp); // test 6/14/21, ok if combined with SampleZUniform(4.0)

      if (ok && acceptanceProbability>Random()) {
	accRatio.Accept();
	if (print) { cout << ss << "ACCEPTED" << ss; Write7(k,j,z,s[k].y,yNew,dy,acceptanceProbability); } // cout << " " << mJ2.p << endl;

	for(int i=0; i<nDim; i++) {
	  s[k].Parameter(i) = sNew.Parameter(i);
	}
	s[k].y = sNew.y; 

      } else {
	if (print) { cout << ss << "REJECTED" << ss; Write7(k,j,z,s[k].y,yNew,dy,acceptanceProbability); } // cout << " " << mJ2.p << endl;
      }

      if (print) { cout << ss << "WALKER " << k << ss << "y= " << s[k].y << " " << s[k] << endl; }
    }
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////

// return values between -pi/2 ... +pi/2 drawn from quadratic distribution, see notes 2/1/20
inline double SampleX() {
  double x = Random();
  double z = asin(x);
  return z;
}

// return values drawn from uniform distribution between -a ... +a
inline double SampleX2(const double a) {
  double x = Random();
  //  double z = 4.0*(x-0.5);
  double z = a*2.0*(x-0.5);
  return z;
}

// Box-Mueller formula - return values drawn from Gaussian distribution with standard deviation 'a'
inline double SampleGaussian(const double a) {
  const double r1 = Random();
  const double r2 = Random();
  return a * sqrt(-2.0*log(r1)) * cos(2.0*pi*r2);
}

inline double LagrangeInterpolation(const double x, const double x0, const double x1, const double x2) { // x0 is special, symmetric in x1 and x2
  return (x-x1)/(x0-x1) * (x-x2)/(x0-x2);
}

template <class S> 
void PerformOneQuadraticMonteCarloSweep(Array1 <S> & s, AccRatio & accRatio, const double a, const int nWalkers, const double temp, const double GaussianFlag) { // do not change 'a'
  if (nWalkers<3) error("nWalkers too small",nWalkers);
  const bool print = false;

  //  int nAccept = 0;
  //  cout << endl << " **************************************" << endl;
  if (print) cout << endl << " Starting quadratic Monte Carlo sweep with " << nWalkers << " walkers." << endl << endl;

  S sNew = s[0]; 
  const int nDim = sNew.nDim();

  int nInvalid = 0;
  for(int i=0; i<nWalkers; i++) { // loop over all walkers
    int j = int(double(nWalkers-1)*Random());
    int k = int(double(nWalkers-2)*Random());
    if (j>=i) { // case j<i, must compare k and with i first. Otherwise k may land on j after being increase because k==i
      j++; // shift some j to avoid j==i
      if (k>=i) k++; // shift some k to avoid k==i
      if (k>=j) k++; // shift some k to avoid k==j
    } else { // case j<i, must compare k and with j first. Otherwise k may land on i after being increase because k==j
      if (k>=j) k++; // shift some k to avoid k==j
      if (k>=i) k++; // shift some k to avoid k==i
    }

    //    double x = SampleX();
    const double tj = -1.0;
    const double tk = +1.0;
    double ti,tNew;
    if (GaussianFlag==false) {
      ti   = SampleX2(a); 
      tNew = SampleX2(a);
    } else {
      ti   = SampleGaussian(a); 
      tNew = SampleGaussian(a);
    }

    const double wi = LagrangeInterpolation(tNew,ti,tj,tk);
    const double wj = LagrangeInterpolation(tNew,tj,tk,ti);
    const double wk = LagrangeInterpolation(tNew,tk,ti,tj);
    if (print) cout << endl;
    if (print) Write8(i,j,k,a,ti,wi,wj,wk);
    //    double z = 1.0; // sNew = s[k]

    for(int d=0; d<nDim; d++) {
      sNew[d] = wi*s[i][d] + wj*s[j][d]+ wk*s[k][d];
    }

    accRatio.Try();
    if (!sNew.Valid()) {

      cout << "Problem: Invalid are the new parameters for walker " << k << endl;
      nInvalid++;
      if (nInvalid>100) error("Something went wrong. Parameters repeatedly invalid.",sNew);

    } else {

      nInvalid = 0;
      bool ok = true;
      double yNew;
      try {
	yNew = sNew.Evaluate();
	if (print) Write8(i,j,k,ti, yNew,s[i].y,s[k].y,s[j].y);
      } 
      catch(const char* message) {
	cout << message << endl;
	sNew.Reset();
	ok = false;
	if (print) cout << "Not ok:"; Write5(i,j,k,ti,sNew); 
      }
      //	if (ok) Write5(i,chi2,chi2New,pOrg,mJ2.p.Parameter(i));
    
      const string ss = " :::: ";
      const double dy = yNew - s[i].y;

      bool accept = false;
      double prob = 0.0;
      if (temp==0.0) {
	if (dy<0.0) {
	  accept = true;
	  prob   = 1.0;
	}
      } else {
	//	double prob = pow(x,nDim-1)*exp(-dy/temp);
	prob   = pow(wi,nDim) * exp(-dy/temp);
	accept = (prob>Random());
      }
      if (ok && accept) {
	accRatio.Accept();
	if (print) { cout << ss << "ACCEPTED" << ss; Write9(i,j,k,ti,s[i].y,yNew,dy,prob,a); }
	//	nAccept++;

	for(int d=0; d<nDim; d++) {
	  s[i][d] = sNew[d];
	}
	s[i].y = sNew.y; 
	//	a *= 1.05; // dangerous

      } else {
	if (print) { cout << ss << "REJECTED" << ss; Write9(i,j,k,ti,s[i].y,yNew,dy,prob,a); } 
	//	a /= 1.05; // dangerous
      }

      //      cout << ss << "WALKER " << i << ss << "y= " << s[i].y << " " << s[i] << endl;
    }
  }
  if (nInvalid>=nWalkers) error("Most or all MC walkers entered invalid parameter region. All new states were invalid.");

}

////////////////////////////////////////////////////////////////////////////////////////////////////

template <class S> 
void PerformWalkMoves(Array1 <S> & s, AccRatio & accRatio, const int nWalkers, const double temp) {
  const bool print = false;
  //  const bool print = true;

  if (print) cout << endl << " Starting sweep of walk moves with " << nWalkers << " walkers." << endl << endl;

  S sNew = s[0]; 
  S sBar = s[0];
  const int nDim = sNew.nDim();

  Array1 <int> index(0,nWalkers,-1); // list of indices of states in subset, initialize elements with -1

  int nInvalid = 0;
  for(int k=0; k<nWalkers; k++) { // loop over all walkers 'k' and try to move them one by one

    index.Resize(0);
    for(int j=0; j<nWalkers; j++) { // loop over all walkers. Construct subset of other walkers.
      if (j!=k) {
	if (Random()>0.5) index.PushBack(j); // select with 50% probability
      }
    }
    if (index.Size()<2) continue; // need at least 2 states in subset

    for(int d=0; d<nDim; d++) { // compute average sBar[d] for all dimensions 'd'
      sBar[d] = 0.0;
      for(int jj=0; jj<index.Size(); jj++) { // loop over all states in subset
	sBar[d] += s[index[jj]][d];
      }
      sBar[d] /= double(index.Size());
    }
    
    for(int d=0; d<nDim; d++) { 
      sNew[d] = s[k][d]; // copy s[k] into sNew 
    }
    for(int jj=0; jj<index.Size(); jj++) { // loop over all states in subset
      const int j = index[jj];
      const double r = BoxMuller();
      //      const double r = RandomRange(-1.0,+1.0);
      if (print) Write3(jj,j,r);
      for(int d=0; d<nDim; d++) { 
	sNew[d] += r * (s[j][d]-sBar[d]);
      }
    }
    if (print) cout << endl;
    if (print) Write2(k,index);

    accRatio.Try();
    if (!sNew.Valid()) {

      cout << "Problem: Invalid are the new parameters for walker " << k << endl;
      nInvalid++;
      if (nInvalid>100) error("Something went wrong. Parameters repeatedly invalid.",sNew);

    } else {
      
      nInvalid = 0; // reset counter 
      bool ok = true;
      double yNew = 0.0;
      try {
	yNew = sNew.Evaluate();
	if (print) Write3(k,yNew,s[k].y);
      } 
      catch(const char* message) {
	cout << message << endl;
	sNew.Reset();
	ok = false;
	if (print) cout << "Not ok:"; Write2(k,sNew); 
	//	if (print) cout << "Not ok:"; Write5(i,j,k,ti,sNew); 
      }
      //	if (ok) Write5(i,chi2,chi2New,pOrg,mJ2.p.Parameter(i));
    
      const string ss = " :::: ";
      const double dy = yNew - s[k].y;

      bool accept = false;
      double prob = 0.0;
      if (dy<0.0) { // always accept downhill moves even if k==kLow
	accept = true;
	prob   = 1.0;
      } else {
	//	prob   = pow(wi,nDim) * exp(-dy/temp); // only for stretch moves
	prob   = exp(-dy/temp);
	accept = (prob>Random());
      }

      if (ok && accept) {
	if (print) { cout << ss << "ACCEPTED" << ss; Write5(k,s[k].y,yNew,dy,prob); }
	//	if (print) { cout << ss << "ACCEPTED" << ss; Write9(i,j,k,ti,s[i].y,yNew,dy,prob,a); }
	accRatio.Accept();
	//	nAccept++;

	for(int d=0; d<nDim; d++) {
	  s[k][d] = sNew[d];
	}
	s[k].y = sNew.y; 

      } else {
	if (print) { cout << ss << "REJECTED" << ss; Write5(k,s[k].y,yNew,dy,prob); } 
      }

      //      cout << ss << "WALKER " << i << ss << "y= " << s[i].y << " " << s[i] << endl;
    }
  }
  if (nInvalid>=nWalkers) error("Most or all MC walkers entered invalid parameter region. All new states were invalid.");
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif // _MC_
